;
; SOFTWARE FOR SD System Z80 Starter System
; to be compiled and installed onto a 2716 eprom put in U33
;
; (c) ESoCoP - Sergio Gervasini - Apr 2019
;
; sds-esocop-prom: 3 little programs to demonstrate an SDS-Z80 system running
; can be compiled with zasm - z80 assembler (c) 1994 - 2019 Günter Woigk.
;  homepage: https://k1.spdns.de/Develop/Projects/zasm/
;
;
; ***** non destructive test memory *****
; check memory with all pattern from 0 to 0xff to all locations
; of the board (2k) without deleting contents
; if the test is successfull at the end the display is 0000
; otherwise it display the addess location and the pattern tested
;
STARTRAM	equ	0x2000
ENDRAM		equ	0x27ff
OUTSEGM		equ	0x88
OUTDIGIT	equ	0x8c

	ORG 	0x0800

	ld bc,STARTRAM
	ld de,ENDRAM
	xor a		; zero acc.
loop1:	ld h,b
	ld l,c
	ex af,af'
	ld a,(hl)	; get actual content and store in alternate reg.set
	ex af,af'
	ld (hl),a	; put pattern test
	cpi		; check a with (hl) [note: hl++ and bc--]
	jr nz,error
	dec hl
	ex af,af'
	ld (hl),a	; restore actual content
	ex af,af'
	inc hl
	ld b,h		; save hl
	ld c,l
	call showhl
	call showa
	sbc hl,de	; ram end?
	jr nz,loop1
	inc a
	jr z,endok
	call showhl
	call showa
	ld bc,STARTRAM
	jr loop1

error:	dec hl
	call showhl
	call showa
	jr error

endok:	call showhl
	jr endok

; show the address in hl registers
showhl:	ex af,af'	; save acc. in alternate register set
	ld a,l
	call outseg
	ld a,4
	call outdgt
	ld a,l
	srl a		; rotate acc. 4 times
	srl a
	srl a
	srl a
	call outseg
	ld a,8
	call outdgt
	ld a,h
	call outseg
	ld a,0x10
	call outdgt
	ld a,h
	srl a		; rotate acc. 4 times
	srl a
	srl a
	srl a
	call outseg
	ld a,0x20
	call outdgt
	call testkey
	ex af,af'	; restore acc. from alternate register set
	ret

; display the content of the acc.
showa:	push af
	call outseg
	ld a,1
	call outdgt
	pop af
	push af
	srl a
	srl a
	srl a
	srl a
	call outseg
	ld a,2
	call outdgt
	pop af
	ret

; pause for 1msec
outdgt:	out (OUTDIGIT),a
	ld a,0x2d	; about 1msec
pause1:	dec a
	jr nz, pause1
	ret

; take the 4 less significant bit of acc. and display the content in 7-segments digit
outseg:	ld ix, segm7
loops:	and 0x0f
	jr z,show
	dec a
	inc ix
	jr loops
show:	ld a,(ix)
	out (OUTSEGM),a
	ret

; check for a key pressed, in this case the system reload
testkey: ld a,0x7f
	out (OUTSEGM),a
	ld a,0x3f
	out (OUTDIGIT),a
	in a,(0x90)
	and 0x1f
	cp 0x1f
	jp nz, 0
	ret

; table bcd -> 7-segment display
segm7:	defb 0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90,0x88,0x83,0xC6,0xA1,0x86,0x8E

;
; ***** clock *****
; demo program to show a clock
;
	ORG 	0x0900

DIVIDER	equ	0570	; to be adjusted to obtain exactly 1 second delay

; initialize registers
	ld b,0		; reg.B : seconds
	ld c,0		; reg.C : minutes
	ld d,0		; reg.D : hours (24h)
	ld e,1		; counter for digit
	ld sp,0x2300	; init stack pointer
	ld hl,DIVIDER

; loop display refresh routine
showc0:	ld a,b		; seconds
showc1:	push af
	call outseg
	ld a,e
	call outdgt
	rlc e
	pop af
	srl a
	srl a
	srl a
	srl a
	call outseg
	ld a,e
	call outdgt
	rlc e
	ld a,e
	and 0xfb	; are we to the 3rd digit?
	jr z,showc
	ld a,e
	and 0xef	; are we to the 5rd digit?
	jr z,showd
	call intr	; increment registers
	ld e,1		; reloop from the 1 digit
	jr showc0
showc:	ld a,c
	jr showc1
showd:	ld a,d
	jr showc1	

intr:	dec l
	jr nz,endi
	dec h		; arrived to 1 sec?
	jr nz,endi
	inc b		; seconds +1
	ld a,b
	and 0x0f
	cp 0x0a		; 10th seconds?
	jr nz,endx
	ld a,b
	and 0xf0
	add a,0x10
	ld b,a
	cp 0x60		; 60th seconds
	jr nz,endx
	xor a		; zero a
	ld b,a
	inc c
	ld a,c
	and 0x0f
	cp 0x0a		; 10th minutes?
	jr nz,endx
	ld a,c
	and 0xf0
	add a,0x10
	ld c,a
	cp 0x60		; 60th minutes?
	jr nz,endx
	xor a
	ld c,a
	inc d
	ld a,d
	sub 0x24	; 24th hour?
	jr z,h24
	ld a,d
	and 0x0f
	cp 0x0a		; 10th hours?
	jr nz,endx
	ld a,d
	and 0xf0
	add a,0x10
	ld d,a
	jr endx
h24:	xor a
	ld d,a
endx:	ld hl,DIVIDER
endi:	ret

;
; ***** running segments *****
; demo program to show a segment at a time on the display
; to simulate a circular motion
; reset to end running
;
	ORG 	0x0A00

start:	ld b,028		; lenght of data
	ld hl,data
loop:	ld a,(hl)
	out (OUTSEGM),a
	inc hl
	ld a,(hl)
	out (OUTDIGIT),a
	ld de,0x30ff
delay:	dec e
	jr nz,delay
	dec d
	jr nz,delay
	inc hl
	dec b
	jr nz,loop
	jr start

; table segment, digit
data:	defb 0x7e,0x20
	defb 0x7e,0x10
	defb 0x7e,0x08
	defb 0x7e,0x04
	defb 0x7e,0x02
	defb 0x7e,0x01
	defb 0x7d,0x01
	defb 0x3f,0x01
	defb 0x3f,0x02
	defb 0x3f,0x04
	defb 0x3f,0x08
	defb 0x3f,0x10
	defb 0x3f,0x20
	defb 0x6f,0x20
	defb 0x77,0x20
	defb 0x77,0x10
	defb 0x77,0x08
	defb 0x77,0x04
	defb 0x77,0x02
	defb 0x77,0x01
	defb 0x7b,0x01
	defb 0x3f,0x01
	defb 0x3f,0x02
	defb 0x3f,0x04
	defb 0x3f,0x08
	defb 0x3f,0x10
	defb 0x3f,0x20
	defb 0x5f,0x20

	end

